# -*- cperl -*- #

=head1 NAME

Xen::Monitor - A module for working with the Xen system

=head1 SYNOPSIS

=for example begin

    #!/usr/bin/perl -w

    use Xen::Monitor;
    use strict;

    my $monitor = Xen::Monitor->new();

    my @available = $monitor->getAvailable();
    my @running   = $monitor->getRunning();
    my @lines     = $monitor->getDMESG();

=for example end


=head1 DESCRIPTION

This module is a simple interface to the Xen monitor.  It allows you
to list the available and running instances of Xen upon the localhost.

The way the module determines available instances is to read files from
the directory /etc/xen with the suffix ".cfg".  Any file which matches
this pattern will be assumed to describe a Xen instance.

The configuration file will then be read to find the name of the available
Xen instance.

Listing running Xen instances is much simpler, and involves invoking
the Xen command "xm --list" to parse the output.

=cut


package Xen::Monitor;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);


require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

($VERSION)       = '$Revision: 1.7 $' =~ m/Revision:\s*(\S+)/;



#
#  Standard modules which we require.
#
use strict;
use warnings;

#
#  Xen-perl modules we use.
#
use Xen::Host;




=head2 new

  Create a new instance of this object.

=cut

sub new
{
    my ( $proto, %supplied ) = (@_);
    my $class = ref($proto) || $proto;

    my $self  = {};

    #
    #  Allow user supplied values to override our defaults
    #
    foreach my $key ( keys %supplied )
    {
	$self->{ lc $key } = $supplied{ $key };
    }

    bless ($self, $class);
    return $self;

}



=head2 getAvailable

  Return an array of all the available Xen virtual instances upon
 this host - that is all Xen instances available whether they are
 running or not.

  We do this by looking for files of the form "/etc/xen/*.cfg", and
 reading the names of each one.

=cut

sub getAvailable
{
    my ( $class ) = ( @_ );

    my @found;

    #
    #  Get the directory that we should load the .cfg files from.
    # this is configurable so that the test function may run.
    #
    my $directory = $class->{'directory'} || "/etc/xen";

    #
    # Read all files in order
    #
    foreach my $file ( glob( $directory . "/*.cfg" ) )
    {
	open( XEN, "<", $file ) or die "Cannot open '$file' - $!";
	
	#
	#  Fall back to the name of the file.
	#
	my $name = $file;
	if ( $name =~ /(.*)\/(.*).cfg/i )
	{
	    $name = $2;
	}

	#
	#  Now attempt to find the real name of this instance.
	#
	while( my $line = <XEN> )
	{
	    if ( $line =~ /name[ \t]*=[ \t]*"(.*)"/ )
	    {
		$name = $1;
	    }
	}
	close( XEN );

	#
	#
	#
	my $new = Xen::Host->new( name => $name );
	push( @found, $new );
    }

    #
    # Return the list.
    #
    return( @found );
}



=head2 getRunning

  Return an array of all the Xen virtual instances currently running
 upon this host.

=cut

sub getRunning
{
    my ( $class ) = ( @_ );

    my @found;

    my $cmd = "xm list  --long 2>/dev/null";

    open( RUNNING, $cmd . "|" ) or die "Error : $!";

    foreach my $line ( <RUNNING> )
    {
	if ( $line =~ /\(name (.*)\)/ )
	{
	    my $name = $1;
	    next if ( $name =~ /^Domain-[0-9]*/ );

	    #
	    #
	    #
	    my $new = Xen::Host->new( name => $name );
	    push( @found, $new );
	}
    }

    #
    # Return the list.
    #
    return( @found );
}




=head2 getDMESG

  Return the 'dmesg' output from the Xen host.

=cut

sub getDMESG
{
    my ( $class ) = ( @_ );

    my $cmd = "xm dmesg  2>/dev/null";
    my @ret = ();

    open( DMESG, $cmd . "|" );
	
    foreach my $line ( <DMESG> )
    {
	push @ret, $line;
    }
    close( DMESG );

    return( @ret );

}


1;
